import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.util.Properties;

import org.apache.jcs.auxiliary.remote.server.RemoteCacheServerFactory;
import org.apache.jcs.engine.control.CompositeCacheManager;
import org.apache.jcs.utils.net.HostNameUtil;
import org.apache.jcs.utils.props.PropertyLoader;
import org.apache.log4j.Logger;

public class JCSServer
{
    static Logger logger = Logger.getLogger("JCSServer");

    private static final int DEFAULT_REGISTRY_PORT = 8888;

    public static void main(String[] args) throws Exception
    {
        final JCSServer cacheManager = new JCSServer();

        cacheManager.startRemoteServer("remote_cache.ccf");
        logger.info("cache service started");

        Runtime.getRuntime().addShutdownHook(new Thread() {
            public void run()
            {
                cacheManager.stopRemoteServer();
                logger.info("cache service stopped");
            }
        });
    }

    public void startRemoteServer(String propsFileName)
    {
        int registryPort = DEFAULT_REGISTRY_PORT;

        try
        {
            Properties props = PropertyLoader.loadProperties(propsFileName);
            if (props != null)
            {
                String portS = props.getProperty("registry.port", String.valueOf(DEFAULT_REGISTRY_PORT));
                try
                {
                    registryPort = Integer.parseInt(portS);
                }
                catch (NumberFormatException e)
                {
                    logger.error("Problem converting port to an int.", e);
                }
            }
        }
        catch (Throwable t)
        {
            logger.error("Problem loading props.", t);
        }

        String registryHost;
        registryHost = HostNameUtil.getLocalHostAddress();

        logger.info("registryHost = [" + registryHost + "]");

        if ("localhost".equals(registryHost) || "127.0.0.1".equals(registryHost))
        {
            logger.warn("The local address [" + registryHost + "] is INVALID.  Other machines must be able to use the address to reach this server.");
        }

        try
        {
            LocateRegistry.createRegistry(registryPort);
        }
        catch (RemoteException e)
        {
            logger.error("Problem creating registry.  It may already be started. " + e.getMessage());
        }
        catch (Throwable t)
        {
            logger.error("Problem creating registry.", t);
        }

        try
        {
            RemoteCacheServerFactory.startup(registryHost, registryPort, "/" + propsFileName);

            logger.info("Remote JCS Server started with properties from " + propsFileName);
        }
        catch (Throwable t)
        {
            logger.error("Problem starting remote cache server.", t);
        }
    }

    public void stopRemoteServer()
    {
        logger.info("dump cached data start");

        CompositeCacheManager.getInstance().shutDown();
    }
}

